@@ -8,7 +8,6 @@ use std::sync::{Arc, Mutex};
 use anyhow::anyhow;
 use base64::encode;
 use maplit::*;
-use serde::{Deserialize, Serialize};
 use serde_json::{json, Value};
 
 use crate::bodies::OptionalBody;
@@ -24,11 +23,11 @@ use crate::sync_interaction::RequestResponseInteraction;
 use crate::v4::async_message::AsynchronousMessage;
 use crate::v4::interaction::V4Interaction;
 use crate::v4::message_parts::MessageContents;
-use crate::v4::sync_message::SynchronousMessages;
+use crate::v4::sync_message::SynchronousMessage;
 use crate::v4::synch_http::SynchronousHttp;
 
 /// Struct that defines a message.
-#[derive(PartialEq, Debug, Clone, Eq, Deserialize, Serialize)]
+#[derive(PartialEq, Debug, Clone, Eq)]
 pub struct Message {
     /// Interaction ID. This will only be set if the Pact file was fetched from a Pact Broker
     pub id: Option<String>,
@@ -37,26 +36,19 @@ pub struct Message {
     pub description: String,
 
     /// Optional provider state for the interaction.
-    /// See https://docs.pact.io/getting_started/provider_states for more info on provider states.
-    #[serde(rename = "providerStates")]
-    #[serde(default)]
+    /// See `<https://docs.pact.io/getting_started/provider_states>` for more info on provider states.
     pub provider_states: Vec<ProviderState>,
 
     /// The contents of the message
-    #[serde(default = "missing_body")]
     pub contents: OptionalBody,
 
     /// Metadata associated with this message.
-    #[serde(default)]
     pub metadata: HashMap<String, Value>,
 
     /// Matching rules
-    #[serde(rename = "matchingRules")]
-    #[serde(default)]
     pub matching_rules: MatchingRules,
 
     /// Generators
-    #[serde(default)]
     pub generators: Generators
 }
 
@@ -89,10 +81,22 @@ impl Interaction for Message {
     self.description.clone()
   }
 
+  fn set_id(&mut self, id: Option<String>) {
+    self.id = id;
+  }
+
+  fn set_description(&mut self, description: &str) {
+    self.description = description.to_string();
+  }
+
   fn provider_states(&self) -> Vec<ProviderState> {
     self.provider_states.clone()
   }
 
+  fn provider_states_mut(&mut self) -> &mut Vec<ProviderState> {
+    &mut self.provider_states
+  }
+
   fn contents(&self) -> OptionalBody {
     self.contents.clone()
   }
@@ -113,6 +117,10 @@ impl Interaction for Message {
     self.as_v4_async_message().map(|i| i.boxed_v4())
   }
 
+  fn as_v4_mut(&mut self) -> Option<&mut dyn V4Interaction> {
+    None
+  }
+
   fn as_v4_http(&self) -> Option<SynchronousHttp> {
     None
   }
@@ -135,15 +143,28 @@ impl Interaction for Message {
     })
   }
 
-  fn as_v4_sync_message(&self) -> Option<SynchronousMessages> {
+  fn as_v4_sync_message(&self) -> Option<SynchronousMessage> {
     None
   }
 
-  fn boxed(&self) -> Box<dyn Interaction + Send> {
+  fn as_v4_http_mut(&mut self) -> Option<&mut SynchronousHttp> {
+    None
+  }
+
+  fn as_v4_async_message_mut(&mut self) -> Option<&mut AsynchronousMessage> {
+    None
+  }
+
+  fn as_v4_sync_message_mut(&mut self) -> Option<&mut SynchronousMessage> {
+    None
+  }
+
+
+  fn boxed(&self) -> Box<dyn Interaction + Send + Sync> {
     Box::new(self.clone())
   }
 
-  fn arced(&self) -> Arc<dyn Interaction + Send> {
+  fn arced(&self) -> Arc<dyn Interaction + Send + Sync> {
     Arc::new(self.clone())
   }
 
@@ -157,21 +178,6 @@ impl Interaction for Message {
 }
 
 impl Message {
-    /// Returns a default message
-    pub fn default() -> Message {
-      Message {
-        id: None,
-        description: "message".to_string(),
-        provider_states: vec![],
-        contents: OptionalBody::Missing,
-        metadata: hashmap!{
-          "contentType".into() => "application/json".into()
-        },
-        matching_rules: MatchingRules::default(),
-        generators: Generators::default()
-      }
-    }
-
     /// Constructs a `Message` from the `Json` struct.
     pub fn from_json(index: usize, json: &Value, spec_version: &PactSpecification) -> anyhow::Result<Message> {
         match spec_version {
@@ -225,7 +231,7 @@ impl Message {
         }
 
         match self.contents {
-          OptionalBody::Present(ref body, _) => {
+          OptionalBody::Present(ref body, _, _) => {
             let content_type = self.message_content_type().unwrap_or_default();
             if content_type.is_json() {
             match serde_json::from_slice(body) {
@@ -272,6 +278,16 @@ impl Message {
       }
     }
   }
+
+  /// Converts this message into a V4 message content
+  pub fn as_message_content(&self) -> MessageContents {
+    MessageContents {
+      contents: self.contents.clone(),
+      metadata: self.metadata.clone(),
+      matching_rules: self.matching_rules.clone(),
+      generators: self.generators.clone()
+    }
+  }
 }
 
 impl HttpPart for Message {
@@ -287,14 +303,26 @@ impl HttpPart for Message {
     &self.contents
   }
 
+  fn body_mut(&mut self) -> &mut OptionalBody {
+    &mut self.contents
+  }
+
   fn matching_rules(&self) -> &MatchingRules {
     &self.matching_rules
   }
 
+  fn matching_rules_mut(&mut self) -> &mut MatchingRules {
+    &mut self.matching_rules
+  }
+
   fn generators(&self) -> &Generators {
     &self.generators
   }
 
+  fn generators_mut(&mut self) -> &mut Generators {
+    &mut self.generators
+  }
+
   fn lookup_content_type(&self) -> Option<String> {
     self.metadata.iter().find(|(k, _)| {
       let key = k.to_ascii_lowercase();
@@ -310,8 +338,20 @@ impl Display for Message {
   }
 }
 
-fn missing_body() -> OptionalBody {
-    OptionalBody::Missing
+impl Default for Message {
+  fn default() -> Self {
+    Message {
+      id: None,
+      description: "message".to_string(),
+      provider_states: vec![],
+      contents: OptionalBody::Missing,
+      metadata: hashmap!{
+          "contentType".into() => "application/json".into()
+        },
+      matching_rules: MatchingRules::default(),
+      generators: Generators::default()
+    }
+  }
 }
 
 #[cfg(test)]
@@ -456,7 +496,7 @@ mod tests {
     }
 
     #[test]
-    fn ignoring_v1_provider_state_when_deserializing_message() {
+    fn v1_provider_state_when_deserializing_message() {
         let message_json = r#"{
             "description": "String",
             "providerState": "provider state",
@@ -464,10 +504,10 @@ mod tests {
             "generators": {}
         }"#;
 
-        // This line should panic, because providerState is not the name of the field.
-        let message: Message = serde_json::from_str(message_json).unwrap();
+        let message_json: Value = serde_json::from_str(message_json).unwrap();
+        let message = Message::from_json(0, &message_json, &PactSpecification::V3).unwrap();
         expect!(message.description).to(be_equal_to("String"));
-        expect!(message.provider_states.iter()).to(be_empty());
+        expect!(message.provider_states.len()).to(be_equal_to(1));
         expect!(message.matching_rules.rules.iter()).to(be_empty());
     }
 
@@ -480,7 +520,8 @@ mod tests {
             "generators": {}
         }"#;
 
-        let message: Message = serde_json::from_str(message_json).unwrap();
+        let message_json: Value = serde_json::from_str(message_json).unwrap();
+        let message = Message::from_json(0, &message_json, &PactSpecification::V3).unwrap();
         expect!(message.description).to(be_equal_to("String"));
         expect!(message.provider_states).to(be_equal_to(vec![ProviderState {
             name: "provider state".to_string(),
@@ -538,7 +579,7 @@ mod tests {
       "AAABMAAAAAAA=");
 
     let mut message = Message::from_json(0, &serde_json::from_str(message_json).unwrap(), &PactSpecification::V3).unwrap();
-    message.contents = OptionalBody::Present(Bytes::from(contents), Some(content_type));
+    message.contents = OptionalBody::Present(Bytes::from(contents), Some(content_type), None);
     let json = message.to_json(&PactSpecification::V3);
 
     expect!(json.get("contents").unwrap().as_str().unwrap()).to(be_equal_to(encoded));
